package ij2x.plugin;

import ij.LookUpTable;
import ij.io.FileInfo;
import ij.ImagePlus;
import ij.IJ;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.File;
import ij.process.ImageProcessor;
import javax.swing.JFrame;
import ij.gui.GenericDialog;

import ij.plugin.filter.PlugInFilter;

import java.awt.Frame;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;

/**
 * <p>Title: Lut2Java</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2011</p>
 *
 * <p>Company: rawak.de</p>
 *
 * @author not attributable
 * @version 1.0
 */
public class Lut2Java implements WindowListener, PlugInFilter{
    private ImagePlus imp;
    private static int index = 1;
    private String fn = "";
    private boolean isShowing=false, windowClosing=false;

    public int setup(String arg, ImagePlus imp) {
            this.imp = imp;
            if (imp!=null) {
                if (imp.getType() == ImagePlus.COLOR_RGB) {
                    IJ.error("Color tables cannot be assiged to RGB Images.");
                    return DONE;
                }
            }else{
                IJ.error("There are no images open.");
                return DONE;
            }
            IJ.register(Lut2Java.class);
            return  DOES_8G | DOES_16 | DOES_32;
    }

    public void run(ImageProcessor ip){
        String dir = "plugins/";
        fn = "Lut_Image_"+ index++;
        isShowing = getString("SaveAs...", "LUT2Java Title: ", fn);
        if(!isShowing)
        return;
        if(fn.equals("")){IJ.beep();index--;return;}
        if(windowClosing)return;
        File file = new File(dir+ fn+".java");
        saveAsJava(file, imp);
    }

    public void saveAsJava(File file, ImagePlus imp) {
            try {
                FileWriter fos = new FileWriter(file);
                    PrintWriter pw = new PrintWriter(fos);
                    saveAsJava(pw, imp);
                    pw.flush();
                    fos.close();
            }
            catch (Exception e) {
                IJ.log("Exception Message: "+e.getMessage());
            }
    }

   /**
    newTitle = getString("FileName...", "Title: ", newTitle);
    */
    boolean getString(String title, String prompt, String defaultString) {
        //JFrame win = imp.getWindow();
        Frame win = imp.getWindow();
            int stackSize = imp.getStackSize();
            fn=defaultString;
            if (win==null)
                    win = IJ.getInstance();
            GenericDialog gd = new GenericDialog(title, win);
            gd.addWindowListener(this);
            gd.addStringField(prompt, defaultString, 20);
            /*if (stackSize>1)
                    gd.addCheckbox("Duplicate Entire Stack", duplicateStack);
            else
                    duplicateStack = false;*/
            gd.showDialog();
            //System.out.println("windowClosing="+windowClosing);
            if (gd.wasCanceled()){
                if(index > 0)index--;
                return false;
            }
            fn = gd.getNextString();
            //if (stackSize>1)
            //        duplicateStack = gd.getNextBoolean();
            return true;
    }

            public void saveAsJava(PrintWriter pw, ImagePlus imp) {
              int g;
              int width=256, height=3;
              LookUpTable lut = imp.createLut();
              byte[] reds = lut.getReds();
              byte[] greens = lut.getGreens();
              byte[] blues = lut.getBlues();
              //short[] pixels = new short[width*height];
              int[][] lutRGB = new int[width][height];
              for (int i=0; i<width; i++) {
                      lutRGB[i][0]=reds[i];
                      lutRGB[i][1]=greens[i];
                      lutRGB[i][2]=blues[i];
              }
              FileInfo fi = new FileInfo();
              fi.width = width;
              fi.height = height;
              //fi.pixels = pixels;

                  pw.println("/**"
                             +"\n* <p>Title: "+fn+".java</p>"
                             +"\n*"
                             +"\n* <p>Description: LUT Image, generated by \"Lut2Java.java\" Converter</p>"
                             +"\n*"
                             +"\n* <p>Copyright: Copyright (c) 2008</p>"
                             +"\n*"
                             +"\n* <p>Company: rawak software</p>"
                             +"\n*"
                             +"\n* @author not attributable"
                             +"\n*"
                             +"\n* @version 1.0"
                             +"\n**/");
                  pw.println("\nimport ij.*;"
                  +"\nimport ij.io.*;"
                  +"\nimport ij.process.*;"
                  +"\nimport javax.swing.JOptionPane;"
                  +"\nimport java.awt.*;"
                  +"\nimport java.io.*;"
                  +"\nimport java.awt.image.*;"
                  +"\nimport java.net.*;\n\n"
                        +"\npublic class "+fn+" extends ij.plugin.LutLoader{\n\n"
                        +"\t\nstatic int width=256, height=3;\n"
                        +"static byte[] reds=new byte[256], greens=new byte[256], blues=new byte[256];\n"
                        +"\t\nstatic short[] pixels = new short[width*height];\n\n"
                        +"\t\n              FileInfo fi = new FileInfo();\n"
                        +"\t\n              JOptionPane pane = new JOptionPane();"
                        );
                  pw.println("\t\n                 public void run(String arg){"
                             +"\t\n                          allLut(reds, greens, blues);"
                             +"\t\n                          fi.width = width;"
                             +"\t\n                          fi.height = height;"
                             +"\t\n                          fi.reds = reds;"
                             +"\t\n                          fi.greens = greens;"
                             +"\t\n                          fi.blues = blues;\n"
                             +"\t\n                          fi.fileName = \""+fn+"\";"
                             +"\t\n                          int iwidth = 256;"
                             +"\t\n                          int iheight = 32;"
                             +"\t\n                          IndexColorModel cm = new IndexColorModel(8, 256, fi.reds, fi.greens, fi.blues);"
                             +"\t\n                          ImagePlus imp = WindowManager.getCurrentImage();"
                             +"\t\n                          if (imp == null){"
                             +"\t\n                          byte[] pixels = new byte[iwidth*iheight];"
                             +"\t\n                          ByteProcessor bp = new ByteProcessor(iwidth, iheight, pixels, cm);"
                             +"\t\n                          int[] ramp = new int[iwidth];"
                             +"\t\n                               for (int i=0; i<iwidth; i++){"
                             +"\t\n                                         ramp[i] = i;}"
                             +"\t\n                          for (int y=0; y<iheight; y++){"
                             +"\t\n                                bp.putRow(0, y, ramp, iwidth);}"
                             +"\t\n                          setProcessor(fi.fileName, bp);"
                             +"\t\n                             show();"
                             +"\t\n                                 }"
                             +"\t\n                             else{"
                             +"\t\n                          if (imp.getType() == ImagePlus.COLOR_RGB) {"
                             +"\t\n                          pane.showConfirmDialog(null, "+"\"Color tables cannot be assiged to RGB Images.\""+", "+"\"ImageJ2x\""+", JOptionPane.CLOSED_OPTION, JOptionPane.INFORMATION_MESSAGE);"
                             +"\t\n                          //IJ.error("+"\"Color tables cannot be assiged to RGB Images.\""+");"
                             +"\t\n                             return;"
                             +"\t\n                                 }"
                             +"\t\n                          ImageProcessor ip = imp.getProcessor();"
                             +"\t\n                          ip.setColorModel((ColorModel)cm);"
                             +"\t\n                          if (imp.getStackSize()>1)"
                             +"\t\n                          imp.getStack().setColorModel((ColorModel)cm);"
                             +"\t\n                          imp.updateAndRepaintWindow();"
                             +"\t\n                                 }"
                             +"\t\n                              }"
                          );

                  pw.println("\t  int allLut(byte[] reds, byte[] greens, byte[] blues) {\n");

                  for (int y=0; y< height; y++) {
                      if(y == 0){
                          pw.print("\t       byte[] red =");
                          pw.print(" {");
                      }
                      else if(y == 1){
                          pw.print("\t          byte[] green =");
                          pw.print(" {");
                      }
                      else if(y == 2){
                          pw.print("\t             byte[] blue =");
                          pw.print(" {");
                      }
                        for (int x=0; x < width-1; x++) {
                                g = lutRGB[x][y];
                                if (g <10) pw.print("  ");
                                else if (g <100) pw.print(" ");
                                pw.print(g+", ");
                        }
                        pw.println(lutRGB[width-1][y]+"};");
                        double percent = (y*1.0/height);
                        IJ.showProgress(percent);
                  }
                  IJ.showProgress(1.0);
                 pw.println("\t\n            for (int i=0; i<red.length; i++) {"
                          +"\t\n                 reds[i] = red[i];"
                          +"\t\n                 greens[i] = green[i];"
                          +"\t\n                 blues[i] = blue[i];"
                          +"\t\n           } return red.length;"
                          +"\t\n         };\t\n       \n}");

            }


            /** Closes this window. */
            public void close() {
                        if(index > 0)
                        {index--;windowClosing=true;}
                        //WindowManager.removeWindow(this);
            }
            public void windowClosing(WindowEvent e) {
                        close();
            }
            public void windowActivated(WindowEvent e) {
                        if (IJ.isMacintosh() && IJ.getInstance()!=null) {
                                IJ.wait(10); // may be needed for Java 1.4 on OS X
                        }
                        windowClosing=false;
                }
            public void windowOpened(WindowEvent e) {}
            public void windowClosed(WindowEvent e) {}
            public void windowIconified(WindowEvent e) {}
            public void windowDeiconified(WindowEvent e) {}
            public void windowDeactivated(WindowEvent e) {}

}
